% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/latex.R
\name{has_latex}
\alias{has_latex}
\alias{check_latex}
\title{Is latex installed?}
\usage{
has_latex()

check_latex()
}
\description{
Checks for presence of pdflatex on path.
}
